/**
 * @module ctt
 * @submodule download-move-in-tool-controller
 */
(function(){
var ContentsTransferApiClient = dixim.ctt.api.ContentsTransferApiClient;
var ContentsTransferToolView = dixim.ctt.ContentsTransferToolView;
var ApplicationToolbarView = dixim.ctt.ApplicationToolbarView;
var ContainerListingController = dixim.ctt.ContainerListingController;
var ContentListingController = dixim.ctt.ContentListingController;
var ContentListingControllerCriteria = dixim.ctt.ContentListingControllerCriteria;
var ErrorDialog = dixim.ctt.ErrorDialog;
var DownloadMoveInWizardController = dixim.ctt.DownloadMoveInWizardController;
var TransferStatusMonitor = dixim.ctt.TransferStatusMonitor;
var TransferProgressController = dixim.ctt.TransferProgressController;
var TransferHistoryController = dixim.ctt.TransferHistoryController;
var TransferScheduleWizardController = dixim.ctt.TransferScheduleWizardController;
var ParentalLockController = dixim.ctt.ParentalLockController;

var DownloadMoveInToolController;
var prototype;

/**
 * Root controller of the Download Move-in Tool.
 *
 * @class DownloadMoveInToolController
 * @constructor
 * @param {Object} configs config object.
 * @param {String} configs.containerNode an ID of the tool container. (ex: '#container', '#wrapper')
 */
DownloadMoveInToolController = function(configs) {
  this._initialize(configs);
};
dixim.ctt.DownloadMoveInToolController = DownloadMoveInToolController;
prototype = DownloadMoveInToolController.prototype;

/**
 * @method _initialize
 * @protected
 * @param {Object} configs config object.
 * @param {String} configs.containerNode
 */
prototype._initialize = function(configs) {
  // TODO: view controller should have sticky flags or
  // something to become a topmost view controller permanently.
  this._modal = null;
  // TODO: ...end.

  this._containerNode = configs.containerNode;
  this._view = new ContentsTransferToolView({containerNode: configs.containerNode});
  this._transferStatusMonitor = new TransferStatusMonitor();

  this._appToolbarView = new ApplicationToolbarView({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_FOOTER_MENU,
    isRemote: true
  });

  this._containerList = new ContainerListingController({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_CONTAINER_LIST,
    model: new ContentsTransferApiClient(),
    isRemote: true
  });

  this._contentList = new ContentListingController({
    containerNode: '#' + ContentsTransferToolView.ELEMENT_ID_CONTENT_LIST,
    model: new ContentsTransferApiClient(),
    isRemote: true
  });
};

/**
 * @method start
 */
prototype.start = function() {
  this._setEventListeners();
  this._view.show();
  this._appToolbarView.show();
  this._containerList.start();
  this._contentList.start();
  this._transferStatusMonitor.start();
};

prototype._setEventListeners = function() {
  this._appToolbarView.addEventListener('unlock', this._didRequestUnlock, this);
  this._appToolbarView.addEventListener('history', this._didRequestHistory, this);
  this._appToolbarView.addEventListener('schedule', this._didRequestSchedule, this);

  this._containerList.addEventListener('selectContainer', this._didSelectContainer, this);
  this._contentList.addEventListener('requestForDownload', this._didRequestForDownload, this);

  this._transferStatusMonitor.addEventListener('transferring', this._didEnterTransferringState, this);
  this._transferStatusMonitor.addEventListener('canceling', this._didEnterTransferringState, this);
};

prototype._didRequestHistory = function() {
  var vc;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferHistoryController();
  vc.addEventListener('close', this._didCloseHistoryDialog, this);
  vc.setPin(this._pin);
  vc.start();
  this._modal = vc;
};

prototype._didCloseHistoryDialog = function(e) {
  this._modal.destroy();
  this._modal = null;
};

prototype._didRequestSchedule = function(e) {
  var vc;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferScheduleWizardController({
    model: new ContentsTransferApiClient({pin: this._pin})
  });
  vc.addEventListener('done', this._didCloseScheduleDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseScheduleDialog = function(e) {
  this._modal.destroy();
  this._modal = null;
};

prototype._didSelectContainer = function(e) {
  var data;
  var args = e.args;

  data = {
    device: args.device,
    containerObject: args.container
  };
  this._contentList.updateCurrentCriteria(new ContentListingControllerCriteria(data));
};

prototype._didRequestForDownload = function(e) {
  var items = e.args.items;
  var device = e.args.device;
  var error;
  var wizard;

  error = this._validateDownloadTarget(items);
  if (!!error) {
    this._showErrorDialog(error);
    return;
  }

  if (!!this._modal) return;

  wizard = new DownloadMoveInWizardController({
    sourceDevice: device,
    targetItems: items,
    model: new ContentsTransferApiClient()
  });
  wizard.addEventListener('done', this._doneDownloadWizard, this);
  wizard.addEventListener('cancel', this._didCancelDownloadWizard, this);
  wizard.start();
  this._modal = wizard;
};

prototype._didEnterTransferringState = function(e) {
  this._startTransferProgress();
};

prototype._didCloseTransferProgress = function(e) {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  this._transferStatusMonitor.start();
  this._reloadListControllers();
};

prototype._doneDownloadWizard = function(e) {
  var data = e.args.registry;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  // XXX: sometimes the transferring has done
  // before the transfer progress UI is opened.
  // Forcibly start the transfer progress UI to
  // show the transferring results.
  if (!data.scheduled) {
    this._startTransferProgress();
    return;
  }
  this._reloadListControllers();
};

prototype._startTransferProgress = function() {
  var vc;

  this._transferStatusMonitor.stop();

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new TransferProgressController({
    model: new ContentsTransferApiClient({pin: this._pin})
  });
  vc.addEventListener('close', this._didCloseTransferProgress, this);
  vc.start();

  this._modal = vc;
};

prototype._didCancelDownloadWizard = function(e) {
  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }
  this._reloadListControllers();
};

prototype._validateDownloadTarget = function(targets) {
  var len = targets.length;
  var i;
  var target;

  if (!len) {
    return 'ctt_error_dialog_download_no_media_selected';
  }

  for (i = 0; i < len; i += 1) {
    target = targets[i];
    if (!target['dnMovable']) {
      return 'ctt_error_dialog_download_unsupported_media_selected';
    }
  }

  return null;
};

prototype._didRequestUnlock = function(e) {
  var vc;

  if (!!this._model) {
    this._modal.destroy();
    this._modal = null;
  }

  vc = new ParentalLockController();
  vc.addEventListener('close', this._didCloseParentalLockDialog, this);
  vc.start();
  this._modal = vc;
};

prototype._didCloseParentalLockDialog = function(e) {
  var pin = e.args.pin;

  if (!!pin) {
    this._appToolbarView.disableUnlockButton();
    this._setPin(pin);
  }
  this._modal.destroy();
  this._modal = null;
};

prototype._setPin = function(pin) {
  this._pin = pin;
  this._didChangePin();
};

prototype._didChangePin = function() {
  var pin = this._pin;

  this._contentList.setPin(pin);
  this._containerList.setPin(pin);
};

/**
 * @method _showErrorDialog
 * @private
 * @param {String} error error ID.
 */
prototype._showErrorDialog = function(error) {
  var dialog;

  if (!!this._modal) {
    this._modal.destroy();
    this._modal = null;
  }

  dialog = new ErrorDialog({type: error});
  dialog.addEventListener('close', this._didCloseModalDialog, this);

  dialog.show();
  this._modal = dialog;
};

prototype._didCloseModalDialog = function(e) {
  var args = e.args;

  if (this._modal === args.dialog) {
    this._modal.destroy();
    this._modal = null;
  }
};

prototype._reloadListControllers = function() {
  this._contentList.reload();
  this._containerList.reload();
};

/**
 * @method stop
 */
prototype.stop = function() {
  this._transferStatusMonitor.stop();
};

}());
