# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

import time
from datetime import datetime

from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django.utils import simplejson
import os
from project.view import FuncView
from project.view import StatusView
from project.view import RootView
from root.modules.form import BaseForm
from project.forms.fields.extras.enable import EnableField
from project.fal.command import get_message
from project.fal.diximdms import FALDiximDMS
from fields import DTCPIPLicenseKeyField

TIMESTAMP_FMT = ":%s"

STATUS_TABLE = {
    "NO_EXEC": _(u"DMS: Status: NO_EXEC"),
    "PROCESSING": _(u"DMS: Status: PROCESSING"),
    "COMPLETE:SUCCESS": _(u"DMS: Status: COMPLETE:SUCCESS"),
    "COMPLETE:ERROR": _(u"DMS: Status: COMPLETE:ERROR"),
    "ABORT": _(u"DMS: Status: ABORT"),
}

def _get_dtcpipinfo():
    try:
        data = FALDiximDMS().query()
        return {
            "needreboot": data["needreboot"],
            "licensekey": data["licensekey"],
        }
    except:
        return {}

class Form(BaseForm):
    def __init__(self, *args, **kwargs):
        from fields import DiximDMSLanguageField
        from fields import DiximDMSShareMultipleCheckboxField

        super(Form, self).__init__(*args, **kwargs)
        self.fields["enable"] = EnableField(label=_(u"DMS: FIELD: LABEL: Enabled"),
                                            initial=True)
        self.fields["language"] = DiximDMSLanguageField()
        self.fields["sharelist"] = DiximDMSShareMultipleCheckboxField()

class DtcpipForm(BaseForm):
    def __init__(self, *args, **kwargs):
        from fields import DiximDMSShareMultipleCheckboxField

        super(DtcpipForm, self).__init__(*args, **kwargs)
        self.fields["sharelist"] = DiximDMSShareMultipleCheckboxField()

class KeyForm(BaseForm):
    key = DTCPIPLicenseKeyField()

class DiximDMSView(FuncView):
    views = ['execute', 'apply', 'upgrade', 'reboot', 'delete', 'dtcpip_apply']

    def view(self, request):
        dtcpip = _get_dtcpipinfo()
        if dtcpip["licensekey"]:
            form = DtcpipForm(initial=self.__initial())
            template = os.path.join(self.TEMPLATES, 'dtcpip.html')
            dictionary = {
                'form': form,
                'licensekey': dtcpip["licensekey"],
            }
        else:
            form = Form(initial=self.__initial())
            template = os.path.join(self.TEMPLATES, 'template.html')
            dictionary = {
                'initialize_status': InitializeStatusView()._status(),
                'form': form,
                'is_needreboot': dtcpip["needreboot"],
                'keyform': KeyForm(),
            }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_execute(self, request):
        form = Form(data=request.POST)

        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()

            result = False
            diximdms = FALDiximDMS()

            if diximdms.initialize(log=True):
                self.set_message(diximdms.get_message())
                result = True

            else:
                self.set_message(diximdms.get_message())

            p.result(result)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'initialize_status': InitializeStatusView()._status(),
            'form': form,
            'is_needreboot': _get_dtcpipinfo()["needreboot"],
            'keyform': KeyForm(),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        form = Form(data=request.POST)
        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()

            if form.is_valid():
                result = False
                data = form.cleaned_data
                diximdms = FALDiximDMS()
                if diximdms.apply({
                    "enable": data["enable"],
                    "language": data["language"],
                    "sharelist": [{"name": name} for name in data["sharelist"]],
                }, log=True):
                    self.set_message(diximdms.get_message())
                    result = True
                else:
                    self.set_message(diximdms.get_message())

            p.result(result)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'initialize_status': InitializeStatusView()._status(),
            'form': form,
            'is_needreboot': _get_dtcpipinfo()["needreboot"],
            'keyform': KeyForm(),
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_upgrade(self, request):
        keyform = KeyForm(data=request.POST)
        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()

            result = False
            if keyform.is_valid():
                cleaned_data = keyform.cleaned_data
                diximdms = FALDiximDMS()
                if diximdms.register_licensekey({
                    "key": cleaned_data['key'],
                }, log=True):
                    self.set_message(diximdms.get_message())
                    result = True
                else:
                    self.set_message(diximdms.get_message())
            else:
                for field in keyform:
                    label = field.label
                    for message in field.errors:
                        self.set_message("%s: %s" % (label, message))

            p.result(result)

        dtcpip = _get_dtcpipinfo()
        if dtcpip["licensekey"]:
            form = DtcpipForm(initial=self.__initial())
            template = os.path.join(self.TEMPLATES, 'dtcpip.html')
            dictionary = {
                'form': form,
                'licensekey': dtcpip["licensekey"],
            }
        else:
            form = Form(initial=self.__initial())
            template = os.path.join(self.TEMPLATES, 'template.html')
            dictionary = {
                'initialize_status': InitializeStatusView()._status(),
                'form': form,
                'is_needreboot': dtcpip["needreboot"],
                'keyform': KeyForm(),
            }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_reboot(self, request):
        from project.fal.power import Power
        form = Form(data=request.POST)
        self.set_message(get_message('MSG_POWER_REBOOT', is_log=True))
        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()
            Power().reboot(force=True)
            p.result(True)

        template = os.path.join(self.TEMPLATES, 'template.html')
        dictionary = {
            'initialize_status': InitializeStatusView()._status(),
            'form': form,
            'is_needreboot': True,
            'keyform': None,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_delete(self, request):
        form = DtcpipForm(data=request.POST)
        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()

            result = False
            diximdms = FALDiximDMS()

            if diximdms.delete_reservation(log=True):
                self.set_message(diximdms.get_message())
                result = True
            else:
                self.set_message(diximdms.get_message())

            p.result(result)

        template = os.path.join(self.TEMPLATES, 'dtcpip.html')
        dictionary = {
            'form': form,
            'licensekey':  _get_dtcpipinfo()["licensekey"],
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_dtcpip_apply(self, request):
        form = DtcpipForm(data=request.POST)
        with self.processing(constname="ERR_DIXIMDMS_STATE_CHANGE") as p:
            p.break_if_state_change()

            if form.is_valid():
                result = False
                data = form.cleaned_data
                diximdms = FALDiximDMS()
                if diximdms.apply({
                    "sharelist": [{"name": name} for name in data["sharelist"]],
                }, log=True):
                    self.set_message(diximdms.get_message())
                    result = True
                else:
                    self.set_message(diximdms.get_message())

            p.result(result)

        template = os.path.join(self.TEMPLATES, 'dtcpip.html')
        dictionary = {
            'form': form,
            'licensekey':  _get_dtcpipinfo()["licensekey"],
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def __initial(self):
        try:
            data = FALDiximDMS().load()
            return {
                "enable": data["enable"],
                "language": data["language"],
                "sharelist": [share["name"] for share in data["sharelist"]],
            }
        except:
            return {}

class InitializeStatusView(StatusView):
    def _status(self):
        try:
            data = FALDiximDMS().query()
            return self._status_info(
                data["status"],
                data["lasttime"]["timestamp"])

        except:
            return ""

    def _status_info(self, status="", timestamp=""):
        status_info = STATUS_TABLE.get(status, "-")

        if status in ["COMPLETE:SUCCESS", "COMPLETE:ERROR"]:
            try:
                string = unicode(datetime(*time.localtime(int(timestamp))[:6]))
                status_info += TIMESTAMP_FMT % string
            except:
                pass

        return status_info


class InitializingView(RootView):
    def view(self, request):
        return {
            self.KEY_RESPONSE: HttpResponse(simplejson.dumps({"initializing": int(self._is_initializing())}),
                                            mimetype="application/json;charset=UTF-8;")
        }

    def _is_initializing(self):
        try:
            data = FALDiximDMS().query()
            status = data["status"]

            return status == "PROCESSING"
        except:

            return False
